@@ -3,7 +3,7 @@
 # An example of creating Excel Column charts with data tables using
 # Python and XlsxWriter.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
 #
 import xlsxwriter
 
@@ -39,9 +39,8 @@ chart1.add_series({
 })
 
 # Configure second series. Note use of alternative syntax to define ranges.
-# List is [ sheet_name, first_row, first_col, last_row, last_col ].
 chart1.add_series({
-    'name':       '=Sheet1!$C$1',
+    'name':       ['Sheet1', 0, 2],
     'categories': ['Sheet1', 1, 0, 6, 0],
     'values':     ['Sheet1', 1, 2, 6, 2],
 })
