@@ -2,11 +2,11 @@
 #
 # An example of adding document properties to a XlsxWriter file.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
 #
-from xlsxwriter.workbook import Workbook
+import xlsxwriter
 
-workbook = Workbook('doc_properties.xlsx')
+workbook = xlsxwriter.Workbook('doc_properties.xlsx')
 worksheet = workbook.add_worksheet()
 
 workbook.set_properties({
