@@ -2,13 +2,13 @@
 #
 # A simple example of merging cells with the XlsxWriter Python module.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
 #
-from xlsxwriter.workbook import Workbook
+import xlsxwriter
 
 
 # Create an new Excel file and add a worksheet.
-workbook = Workbook('merge1.xlsx')
+workbook = xlsxwriter.Workbook('merge1.xlsx')
 worksheet = workbook.add_worksheet()
 
 # Increase the cell size of the merged cells to highlight the formatting.
