@@ -2,7 +2,7 @@
 #
 # Workbook - A class for writing the Excel XLSX Workbook file.
 #
-# Copyright 2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2014, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
@@ -104,6 +104,9 @@ class Workbook(xmlwriter.XMLwriter):
         self.border_count = 0
         self.fill_count = 0
         self.drawing_count = 0
+        self.calc_mode = "auto"
+        self.calc_on_load = True
+        self.allow_zip64 = False
 
         # We can't do 'constant_memory' mode while doing 'in_memory' mode.
         if self.in_memory:
@@ -215,6 +218,8 @@ class Workbook(xmlwriter.XMLwriter):
             chart.chart_name = options['name']
 
         chart.embedded = True
+        chart.date_1904 = self.date_1904
+
         self.charts.append(chart)
 
         return chart
@@ -247,6 +252,27 @@ class Workbook(xmlwriter.XMLwriter):
         """
         self.doc_properties = properties
 
+    def set_calc_mode(self, mode):
+        """
+        Set the Excel caclcuation mode for the workbook.
+
+        Args:
+            mode: String containing one of:
+                * manual
+                * auto_except_tables
+                * auto
+
+        Returns:
+            Nothing.
+
+        """
+        self.calc_mode = mode
+
+        if mode == 'manual':
+            self.calc_on_load = False
+        elif mode == 'auto_except_tables':
+            self.calc_mode = 'autoNoTable'
+
     def define_name(self, name, formula):
         # Create a defined name in Excel. We handle global/workbook level
         # names and local/worksheet names.
@@ -318,6 +344,19 @@ class Workbook(xmlwriter.XMLwriter):
         """
         return self.worksheets_objs
 
+    def use_zip64(self):
+        """
+        Allow ZIP64 extensions when writing xlsx file zip container.
+
+        Args:
+            None.
+
+        Returns:
+            Nothing.
+
+        """
+        self.allow_zip64 = True
+
     ###########################################################################
     #
     # Private API.
@@ -405,14 +444,18 @@ class Workbook(xmlwriter.XMLwriter):
         # Free up the Packager object.
         packager = None
 
-        xlsx_file = ZipFile(self.filename, "w", compression=ZIP_DEFLATED)
+        xlsx_file = ZipFile(self.filename, "w", compression=ZIP_DEFLATED,
+                            allowZip64=self.allow_zip64)
 
         # Add XML sub-files to the Zip file with their Excel filename.
-        for os_filename, xml_filename in xml_files:
+        for os_filename, xml_filename, is_binary in xml_files:
             if self.in_memory:
                 # The files are in-memory StringIOs.
-                xlsx_file.writestr(xml_filename,
-                                   os_filename.getvalue().encode('utf-8'))
+                if is_binary:
+                    xlsx_file.writestr(xml_filename, os_filename.getvalue())
+                else:
+                    xlsx_file.writestr(xml_filename,
+                                       os_filename.getvalue().encode('utf-8'))
             else:
                 # The files are tempfiles.
                 xlsx_file.write(os_filename, xml_filename)
@@ -585,28 +628,23 @@ class Workbook(xmlwriter.XMLwriter):
                 xf_format.has_dxf_font = 1
 
     def _prepare_num_formats(self):
-        # User records is not None start from index 0xA4.
+        # User defined records in Excel start from index 0xA4.
         num_formats = {}
         index = 164
         num_format_count = 0
 
-        is_number = re.compile(r'^\d+$')
-        is_zeroes = re.compile(r'^0+\d')
-
         for xf_format in (self.xf_formats + self.dxf_formats):
             num_format = xf_format.num_format
-            # Check if num_format is an index to a built-in number format.
-            # Also check for a string of zeros, which is a valid number
-            # format string but would evaluate to zero.
 
-            try:
-                if (is_number.match(str(num_format))
-                        and not is_zeroes.match(str(num_format))):
-                    # Index to a built-in number xf_format.
-                    xf_format.num_format_index = int(num_format)
-                    continue
-            except (TypeError, UnicodeEncodeError):
-                pass
+            if sys.version_info[0] == 2:
+                str_types = basestring
+            else:
+                str_types = str
+
+            # Check if num_format is an index to a built-in number format.
+            if not isinstance(num_format, str_types):
+                xf_format.num_format_index = int(num_format)
+                continue
 
             if num_format in num_formats:
                 # Number xf_format has already been used.
@@ -772,9 +810,10 @@ class Workbook(xmlwriter.XMLwriter):
 
             name_list.append(defined_name + "::" + sheet_name)
 
-        # Remove the extra key for sorting.
+        # Sort based on the normalised key.
         names.sort(key=operator.itemgetter(4))
 
+        # Remove the extra key used for sorting.
         for name_list in names:
             name_list.pop()
 
@@ -1038,11 +1077,22 @@ class Workbook(xmlwriter.XMLwriter):
                 if sheetname is None:
                     continue
 
-                # Die if the name is unknown since it indicates a user error in
-                # a chart series formula.
+                # Handle non-contiguous ranges like:
+                #     (Sheet1!$A$1:$A$2,Sheet1!$A$4:$A$5).
+                # We don't try to parse them. We just return an empty list.
+                if sheetname.startswith('('):
+                    chart.formula_data[r_id] = []
+                    seen_ranges[c_range] = []
+                    continue
+
+                # Warn if the name is unknown since it indicates a user error
+                # in a chart series formula.
                 if not sheetname in worksheets:
                     warn("Unknown worksheet reference '%s' in range "
                          "'%s' passed to add_series()" % (sheetname, c_range))
+                    chart.formula_data[r_id] = []
+                    seen_ranges[c_range] = []
+                    continue
 
                 # Find the worksheet object based on the sheet name.
                 worksheet = worksheets[sheetname]
@@ -1050,10 +1100,7 @@ class Workbook(xmlwriter.XMLwriter):
                 # Get the data from the worksheet table.
                 data = worksheet._get_range_data(*cells)
 
-                # TODO
-                #   # Ignore rich strings for now. Deparse later if necessary.
-                #        if token =~ m{^<r>} and token =~ m{</r>$}:
-                #            token = ''
+                # TODO. Handle SST string ids if required.
 
                 # Add the data to the chart.
                 chart.formula_data[r_id] = data
@@ -1069,13 +1116,13 @@ class Workbook(xmlwriter.XMLwriter):
         # TODO. Fix this to match from right.
         pos = c_range.find('!')
         if pos > 0:
-            sheetname, cells = c_range.split('!')
+            sheetname, cells = c_range.split('!', 1)
         else:
             return None
 
         # Split the cell range into 2 cells or else use single cell for both.
         if cells.find(':') > 0:
-            (cell_1, cell_2) = cells.split(':')
+            (cell_1, cell_2) = cells.split(':', 1)
         else:
             (cell_1, cell_2) = (cells, cells)
 
@@ -1208,8 +1255,16 @@ class Workbook(xmlwriter.XMLwriter):
 
     def _write_calc_pr(self):
         # Write the <calcPr> element.
-        attributes = [('calcId', '124519'),
-                      ('fullCalcOnLoad', '1')]
+        attributes = [('calcId', '124519')]
+
+        if self.calc_mode == 'manual':
+            attributes.append(('calcMode', self.calc_mode))
+            attributes.append(('calcOnSave', "0"))
+        elif self.calc_mode == 'autoNoTable':
+            attributes.append(('calcMode', self.calc_mode))
+
+        if self.calc_on_load:
+            attributes.append(('fullCalcOnLoad', '1'))
 
         self._xml_empty_tag('calcPr', attributes)
 
